% File src/library/utils/man/normalizePath.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2014 R Core Team
% Distributed under GPL 2 or later

\name{normalizePath}
\alias{normalizePath}
\title{Express File Paths in Canonical Form}
\description{
  ファイルパスをプラットフォームに対する標準系に変換し，
  それらをユーザが理解できる形で提示するので
  相対と絶対パスを比較できる．
}
\usage{
normalizePath(path, winslash = "\\\\", mustWork = NA)
}
\arguments{
  \item{path}{ファイルパスの文字列ベクトル．}
  \item{winslash}{Windows で使われる分離記号 -- 他では無視される．
    \code{c("/", "\\\\")} のどれかである必要がある．}
  \item{mustWork}{論理値：もし \code{TRUE} なら，もし結果が決定できなければ
    エラーが与えられる；もし \code{NA} ならば警告．}
}
\details{
  チルド展開(\code{\link{path.expand}} を見よ)が \code{paths} に
  関してされる．

  Unix 風プラットフォームがそれをサポートしていれば，これはパスを
  標準形の絶対パス(\samp{./}, \samp{../} そしてシンボリックリンクが無い)
  に変換する．これは POSIX システム関数 \code{realpath} に依存している：
  もしプラットフォームがそれを持たなければ(現在の例は知らない)
  結果は絶対パスであるが標準形ではないかもしれない．
  \code{realpath} を使ったとしても標準形はユニークではないかもしれない．
  例えばハードリンクや多重マウントの場合である．

  Windows ではこれは相対パスを絶対パスに変換し，
  パス要素の短縮名を長い名前に変換し，そして分離記号を
  \code{winslash} で指定されたものにする．それは大・小文字を区別せずに
  パスにマッチし，標準ケースを返す．
  現在のロケールで不正な UTF-8 でエンコードされたパスを使うことが出来る．

  \code{mustWork = FALSE} はメッセージ中で使うためにパスを表現するのに
  有用である．
}

\value{
  文字列ベクトル．

  もし入力が実在のパスでなければ結果はシステム依存である(
  \code{mustWork = TRUE} で無い限り，その場合はエラーになるはずである)．それは
  対応するパス要素かそれを絶対パスに変換したものになる．

  絶対パスへの変換は多くの理由から失敗する可能性がある．
  最も普通の理由は
  \itemize{
    \item ファイルパスの一つ以上の成分が存在しない．
    \item 最後の手前の成分がディレクトリでない，または
    このディレクトリを読む十分なパーミッションが無い．
    \item 相対パスに対しては，現在のディレクトリを決定できない．
    \item シンボリックリンクが存在しない場所を指しているか，
    リンクがループしている．
    \item 標準化されたパスがファイルパスにサポートされる最大の
    長さを超える．
  }
}

#ifdef windows
\seealso{
  \code{\link{shortPathName}}
}
#endif

\examples{\donttest{ # ランダムな一時ディレクトリ
cat(normalizePath(c(R.home(), tempdir())), sep = "\n")
}}
\keyword{ utilities }

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/utils/man/normalizePath.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2014 R Core Team
%% Distributed under GPL 2 or later
%
%\name{normalizePath}
%\alias{normalizePath}
%\title{Express File Paths in Canonical Form}
%\description{
%  Convert file paths to canonical form for the platform, to display them
%  in a user-understandable form and so that relative and absolute paths can
%  be compared.
%}
%\usage{
%normalizePath(path, winslash = "\\\\", mustWork = NA)
%}
%\arguments{
%  \item{path}{character vector of file paths.}
%  \item{winslash}{the separator to be used on Windows -- ignored
%    elsewhere.  Must be one of \code{c("/", "\\\\")}.}
%  \item{mustWork}{logical: if \code{TRUE} then an error is given if the result
%    cannot be determined; if \code{NA} then a warning.}
%}
%\details{
%  Tilde-expansion (see \code{\link{path.expand}}) is first done on
%  \code{paths}.
%
%  Where the Unix-alike platform supports it attempts to turn paths into
%  absolute paths in their canonical form (no \samp{./}, \samp{../} nor
%  symbolic links).  It relies on the POSIX system function
%  \code{realpath}: if the platform does not have that (we know of no
%  current example) then the result will be an absolute path but might
%  not be canonical.  Even where \code{realpath} is used the canonical
%  path need not be unique, for example \emph{via} hard links or
%  multiple mounts.
%
%  On Windows it converts relative paths to absolute paths, converts
%  short names for path elements to long names and ensures the separator
%  is that specified by \code{winslash}.  It will match paths
%  case-insensitively and return the canonical case.
%  UTF-8-encoded paths not valid in the current locale can be used.
%
%  \code{mustWork = FALSE} is useful for expressing paths for use in
%  messages.
%}
%
%\value{
%  A character vector.
%
%  If an input is not a real path the result is system-dependent (unless
%  \code{mustWork = TRUE}, when this should be an error).  It will be
%  either the corresponding input element or a transformation of it into
%  an absolute path.
%
%  Converting to an absolute file path can fail for a large number of
%  reasons.  The most common are
%  \itemize{
%    \item One of more components of the file path does not exist.
%    \item A component before the last is not a directory, or there is
%    insufficient permission to read the directory.
%    \item For a relative path, the current directory cannot be
%    determined.
%    \item A symbolic link points to a non-existent place or links form a
%    loop.
%    \item The canonicalized path would be exceed the maximum supported
%    length of a file path.
%  }
%}
%
%#ifdef windows
%\seealso{
%  \code{\link{shortPathName}}
%}
%#endif
%
%\examples{\donttest{ # random tempdir
%cat(normalizePath(c(R.home(), tempdir())), sep = "\n")
%}}
%\keyword{ utilities }
%
\encoding{UTF8}
