% File src/library/base/man/sweep.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2010 R Core Team
% Distributed under GPL 2 or later

\name{sweep}
\alias{sweep}
\title{Sweep out Array Summaries}
\description{
  入力配列から要約統計量をスイープ(順次適用)することで得られる配列を返す．
}
\usage{
sweep(x, MARGIN, STATS, FUN = "-", check.margin = TRUE, \dots)
}
\arguments{
  \item{x}{配列．}
  \item{MARGIN}{\code{STATS} に対応する \code{x} の を与える添字ベクトル．}
  \item{STATS}{スイープされる要約統計量．}
  \item{FUN}{スイープを実行するために使われる関数．}
  \item{check.margin}{論理値．もし \code{TRUE} (既定値)ならば
    \code{STATS} の長さや次元が \code{x} の指定された次元に一致しなければ
    警告する．}
  \item{\dots}{\code{FUN} への追加引数．}
}
\details{
  \code{FUN} は \code{\link{match.fun}} の呼び出しで見つけられる．
  既定に於けるようにバイナリ演算子が引用化するか逆引用化すれば提供できる．

  \code{FUN} は二つの引数をもつ関数でなければならない：
  それは引数 \code{x} と \code{\link{aperm}} により \code{STATS}
  から生成された同じ次元の配列を使って呼び出される．

  \code{STATS} がベクトルであるよりも配列であるならば，
  \code{STATS}, \code{MARGIN} そして \code{x} 間の一貫性チェックはより
  厳密になる．ベクトルのケースでは警告無しにある種のリサイクルが許される．
  もし \code{STATS} がベクトルで，何らかのリサイクルが行われたら
  警告を必要とすれば \code{sweep(x, MARGIN, as.array(STATS))} を
  使う．
}
\value{
  \code{x} と同じ形の配列であるが，要約統計量でスイープされている．
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
}
\seealso{
  \code{sweep} が基礎としていた \code{\link{apply}}．
  センタリングとスケーリングには \code{\link{scale}}．
}
\examples{
require(stats) # 中央値に対して
med.att <- apply(attitude, 2, median)
sweep(data.matrix(attitude), 2, med.att)  # 列中央値を引き去る

## より一層のスイープ：
A <- array(1:24, dim = 4:2)

## 普通の使用では警告はない
sweep(A, 1, 5)
(A.min <- apply(A, 1, min))  # == 1:4
sweep(A, 1, A.min)
sweep(A, 1:2, apply(A, 1:2, median))

## ミスマッチには警告
sweep(A, 1, 1:3)  # STATS はリサイクルされない
sweep(A, 1, 6:1)  # STATS がより長い

## 正確なリサイクル：
sweep(A, 1, 1:2)  # 警告なし
sweep(A, 1, as.array(1:2))  # 警告
}
\keyword{array}
\keyword{iteration}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/sweep.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2010 R Core Team
%% Distributed under GPL 2 or later
%
%\name{sweep}
%\alias{sweep}
%\title{Sweep out Array Summaries}
%\description{
%  Return an array obtained from an input array by sweeping out a summary
%  statistic.
%}
%\usage{
%sweep(x, MARGIN, STATS, FUN = "-", check.margin = TRUE, \dots)
%}
%\arguments{
%  \item{x}{an array.}
%  \item{MARGIN}{a vector of indices giving the extent(s) of \code{x}
%    which correspond to \code{STATS}.}
%  \item{STATS}{the summary statistic which is to be swept out.}
%  \item{FUN}{the function to be used to carry out the sweep.}
%  \item{check.margin}{logical.  If \code{TRUE} (the default), warn if the
%    length or dimensions of \code{STATS} do not match the specified
%    dimensions of \code{x}.  Set to \code{FALSE} for a small speed gain
%    when you \emph{know} that dimensions match.}
%  \item{\dots}{optional arguments to \code{FUN}.}
%}
%\details{
%  \code{FUN} is found by a call to \code{\link{match.fun}}.  As in the
%  default, binary operators can be supplied if quoted or backquoted.
%
%  \code{FUN} should be a function of two arguments: it will be called
%  with arguments \code{x} and an array of the same dimensions generated
%  from \code{STATS} by \code{\link{aperm}}.
%
%  The consistency check among \code{STATS}, \code{MARGIN} and \code{x}
%  is stricter if \code{STATS} is an array than if it is a vector.
%  In the vector case, some kinds of recycling are allowed without a
%  warning.  Use \code{sweep(x, MARGIN, as.array(STATS))} if \code{STATS}
%  is a vector and you want to be warned if any recycling occurs.
%}
%\value{
%  An array with the same shape as \code{x}, but with the summary
%  statistics swept out.
%}
%\references{
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%}
%\seealso{
%  \code{\link{apply}} on which \code{sweep} used to be based;
%  \code{\link{scale}} for centering and scaling.
%}
%\examples{
%require(stats) # for median
%med.att <- apply(attitude, 2, median)
%sweep(data.matrix(attitude), 2, med.att)  # subtract the column medians
%
%## More sweeping:
%A <- array(1:24, dim = 4:2)
%
%## no warnings in normal use
%sweep(A, 1, 5)
%(A.min <- apply(A, 1, min))  # == 1:4
%sweep(A, 1, A.min)
%sweep(A, 1:2, apply(A, 1:2, median))
%
%## warnings when mismatch
%sweep(A, 1, 1:3)  # STATS does not recycle
%sweep(A, 1, 6:1)  # STATS is longer
%
%## exact recycling:
%sweep(A, 1, 1:2)  # no warning
%sweep(A, 1, as.array(1:2))  # warning
%}
%\keyword{array}
%\keyword{iteration}
%