% File src/library/base/man/unlist.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2013 R Core Team
% Distributed under GPL 2 or later

\name{unlist}
\title{Flatten Lists}
\description{
  リスト構造 \code{x} をあたえた時，\code{unlist} は \code{x} 中の全ての
  原子的要素を含むベクトルを作ることでそれを簡略化する．
}
\usage{
unlist(x, recursive = TRUE, use.names = TRUE)
}
\alias{unlist}
\arguments{
  \item{x}{\R オブジェクト．典型的にはリスト化ベクトル．}
  \item{recursive}{論理値．非リスト化操作は \code{x} のリスト成分にも
    適用すべきか?}
  \item{use.names}{論理値．名前を保存すべきか?}
}
\details{
  \code{unlist} は総称的である：特定のクラスのオブジェクトを処理するメソッドを
  書くことが出来る，\link{InternalMethods} を見よ．
  そして，例えば \code{relistable} オブジェクトに対する \code{unlist} メソッドを持つ
  \code{\link{relist}} に注意しよう．

  もし \code{recursive = FALSE} ならば，関数は \code{x} 中の最初の第一レベルの項目を
  超えて再帰的に作用しない．

  因子は特別に扱われる．もし \code{x} の全てのリストでない要素が
  因子(又は順序付き因子)ならば要素の因子レベルの集合の合併を水準に持つ
  因子で，水準は要素の水準集合に登場する順序になる
  (これは全ての要素が同じ水準集合を持てばそれが結果の水準集合になることを
  意味する)．

  \code{x} は原子的なベクトルでも良いが，そうすると \code{unlist} は役に立つ
  何も行わず，名前すら捨て去らない．

  既定では \code{unlist} は \code{x} 中に存在する名前情報を保とうと試みる．
  もし \code{use.names = FALSE} ならば全ての名前情報は捨て去られる．

  可能な箇所では非リスト化の過程でリスト要素は同じモードに強制変換されるので，
  結果はしばしば文字列ベクトルになってしまう．
  ベクトルは階層 NULL < raw < logical < integer < double < complex < character
  < list < expression で最高位の成分の型に強制変換される：
  対リストはリストとして扱われる．

  リストは(総称的な)ベクトルで，簡略化されたベクトルは依然リストかもしれない
  (そして変更されないかもしれない)．リストのベクトルでない要素
  (例えば名前，公式，そして呼び出しのような言語要素)
  は強制変換されないので，これらを一つ以上含むリストは
  リストに留まる．(\code{\link{lm}} による当てはめの非リスト化の効果は
  各残差を成分に持つリストである．)
  \code{unlist(x)} はここではエラーにする代わりにベクトルでない
  \code{x} に対しても未変更で
  \code{x} を返すことを注意する．
}
\value{
  \code{NULL} 又は表現式又はリスト成分を含む適当なモードのベクトル．

  出力の型は対リストをリストに強制変換した後の階層
  NULL < raw < logical < integer < double <
  complex < character < list < expression での最高位の型から決定される．
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.
}
\seealso{
  \code{\link{c}}, \code{\link{as.list}}, \code{\link{relist}}.
}
\examples{
unlist(options())
unlist(options(), use.names = FALSE)

l.ex <- list(a = list(1:5, LETTERS[1:5]), b = "Z", c = NA)
unlist(l.ex, recursive = FALSE)
unlist(l.ex, recursive = TRUE)

l1 <- list(a = "a", b = 2, c = pi+2i)
unlist(l1) # 文字列ベクトル
l2 <- list(a = "a", b = as.name("b"), c = pi+2i)
unlist(l2) # リストのまま

ll <- list(as.name("sinc"), quote( a + b ), 1:10, letters, expression(1+x))
utils::str(ll)
for(x in ll)
  stopifnot(identical(x, unlist(x)))
}
\keyword{list}
\keyword{manip}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/unlist.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2013 R Core Team
%% Distributed under GPL 2 or later
%
%\name{unlist}
%\title{Flatten Lists}
%\description{
%  Given a list structure \code{x}, \code{unlist} simplifies it to
%  produce a vector which contains all the atomic components
%  which occur in \code{x}.
%}
%\usage{
%unlist(x, recursive = TRUE, use.names = TRUE)
%}
%\alias{unlist}
%\arguments{
%  \item{x}{an \R object, typically a list or vector.}
%  \item{recursive}{logical.  Should unlisting be applied to list
%    components of \code{x}?}
%  \item{use.names}{logical.  Should names be preserved?}
%}
%\details{
%  \code{unlist} is generic: you can write methods to handle
%  specific classes of objects, see \link{InternalMethods},
%  and note, e.g., \code{\link{relist}} with the \code{unlist} method
%  for \code{relistable} objects.
%
%  If \code{recursive = FALSE}, the function will not recurse beyond the
%  first level items in \code{x}.
%
%  Factors are treated specially.  If all non-list elements of \code{x}
%  are factors (or ordered factors) then the result will be a factor with
%  levels the union of the level sets of the elements, in the order the
%  levels occur in the level sets of the elements (which means that if
%  all the elements have the same level set, that is the level set of the
%  result).
%
%  \code{x} can be an atomic vector, but then \code{unlist} does nothing useful,
%  not even drop names.
%
%  By default, \code{unlist} tries to retain the naming
%  information present in \code{x}.  If \code{use.names = FALSE} all
%  naming information is dropped.
%
%  Where possible the list elements are coerced to a common mode during
%  the unlisting, and so the result often ends up as a character
%  vector.  Vectors will be coerced to the highest type of the components
%  in the hierarchy NULL < raw < logical < integer < double < complex < character
%  < list < expression: pairlists are treated as lists.
%
%  A list is a (generic) vector, and the simplified vector might still be
%  a list (and might be unchanged).  Non-vector elements of the list
%  (for example language elements such as names, formulas and calls)
%  are not coerced, and so a list containing one or more of these remains a
%  list.  (The effect of unlisting an \code{\link{lm}} fit is a list which
%  has individual residuals as components.)
%  Note that \code{unlist(x)} now returns \code{x} unchanged also for
%  non-vector \code{x}, instead of signalling an error in that case.
%}
%\value{
%  \code{NULL} or an expression or a vector of an appropriate mode to
%  hold the list components.
%
%  The output type is determined from the highest type
%  of the components in the hierarchy NULL < raw < logical < integer < double <
%  complex < character < list < expression, after coercion of pairlists
%  to lists.
%}
%\references{
%  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988)
%  \emph{The New S Language}.
%  Wadsworth & Brooks/Cole.
%}
%\seealso{
%  \code{\link{c}}, \code{\link{as.list}}, \code{\link{relist}}.
%}
%\examples{
%unlist(options())
%unlist(options(), use.names = FALSE)
%
%l.ex <- list(a = list(1:5, LETTERS[1:5]), b = "Z", c = NA)
%unlist(l.ex, recursive = FALSE)
%unlist(l.ex, recursive = TRUE)
%
%l1 <- list(a = "a", b = 2, c = pi+2i)
%unlist(l1) # a character vector
%l2 <- list(a = "a", b = as.name("b"), c = pi+2i)
%unlist(l2) # remains a list
%
%ll <- list(as.name("sinc"), quote( a + b ), 1:10, letters, expression(1+x))
%utils::str(ll)
%for(x in ll)
%  stopifnot(identical(x, unlist(x)))
%}
%\keyword{list}
%\keyword{manip}
%